#=========================================================
# ColorListDuplicator
#
# Blender2.41
# Python Release 2.4.3 29 March 2006
#
# Auther: keina
#
# History:
#   2006.05.02(Tue)  ButtonEvent 266:CommentOut
#   2006.04.28(Fri)  Created
#
# CopyRight(c) 2006 K.Keina
# Mail: keines@ps.sakura.ne.jp
# Web : http://keines.gozaru.jp/blender/
#=========================================================
import Blender,re,sys,time
from Blender import Draw, BGL, Window, Scene, Material

# Event Numbers
Event_Exit	= 0
Event_Select	= 1
Event_Input	= 2
Event_Column	= 3
Event_StepX	= 4
Event_StepY	= 5
Event_Duplicate	= 6
Event_Menu	= 7

# Initial values buttons
InputBtn	= Draw.Create('')	# FilePath
ColumnBtn	= Draw.Create(12)
StepXBtn	= Draw.Create(2.0)
StepYBtn	= Draw.Create(2.0)
MenuBtn		= Draw.Create(1)

# Other variables
Title		= 'ColorList Duplicator'
Version		= 'ver.2006.05.01(Mon)'
Author		= ' by keines'
BaseName	= 'Mat.%03d'

# 0:Decimal,Hex  : D9,00
# 1:Html         : 01ABCF,0AB or #01ABCF,#0AB
RegExp=["([^\t]*)\t(\w+)\t(\w+)\t(\w+)$","([^\t]*)\t#?(\w\w?)(\w\w?)(\w\w?)$"]

#=========================================================
# Functions
#=========================================================
def ConfirmPopMenu(s):
    return Draw.PupMenu('Confirm.. %t|'+s+' %x1')

def ExitFunction():
    if ConfirmPopMenu('Exit Script ?')==1:
        print 'Exit.'
        Draw.Exit()
        return

def FileSelectFunction(fn):
    InputBtn.val=fn

def DuplicateFunction():
    #try:
    #except :
    #  print "  error.." 
    #  break
    #print Blender.sys.exists(InputBtn.val)
    #return
    
    if not (Blender.sys.exists(InputBtn.val)==1):
        ConfirmPopMenu('File not Found.')
        return
    
    scn = Scene.GetCurrent()
    activeObject = scn.getActiveObject()
    
    if not activeObject:
        ConfirmPopMenu('ActiveObject not Found.')
        return
    
    print '    start\n','        ',InputBtn.val
    
    ### Unselect all ###
    for ob in Blender.Object.GetSelected():
        ob.sel = 0
    activeObject.sel = 1
    
    ### Material List ###
    work=Material.Get()
    
    ### Base Material Check ###
    i="Material"
    me=activeObject.getData()
    if not me.materials:
        for mat in work:
            if mat.name==i:
                i=""
                break
    if i<>"":
        mat=Material.New(i)
    me.materials=[mat]
    me.update()
    
    ### Init ###
    # ẅړ
    w=StepXBtn.val
    # ẍړ
    h=StepYBtn.val
    
    BaseX=activeObject.LocX+w
    BaseY=activeObject.LocY
    cnt=-1
    R=0
    G=0
    B=0
    
    start=time.time()
    try:
        ### Loop Read & Duplicate ###
        #print '- Material Start -'
        f=open(InputBtn.val)
        # skip header
        f.readline()
        # Html=RegExp[1]
        rexp=re.compile(RegExp[(MenuBtn.val==3)])
        for s in f.readlines():
            cnt+=1
            #n=rexp.split(s)
            n=rexp.findall(s)
            if not n: continue
            n=n.pop(0)
            if (len(n[0])==0):n[0]=BaseName % (cnt+1)
            # Search Name
            k=1
            for mat in work:
                if mat.name==n[0]:
                    k=0
                    break
            # Duplicate & Move  k:1=new
            Blender.Object.Duplicate(mesh=1,material=k)
            activeObject = scn.getActiveObject()
            activeObject.LocX=BaseX+(cnt%ColumnBtn.val)*w+w
            activeObject.LocY=BaseY-(cnt/ColumnBtn.val)*h
            # GetNMesh
            me=activeObject.getData()
            # 1:Decimal 2:Hex 3:Html
            if (MenuBtn.val==1):
                R=int(n[1],10)
                G=int(n[2],10)
                B=int(n[3],10)
            else:
                # Str(Hex) -> Int(Decimal)
                R=int(n[1], 16)
                G=int(n[2], 16)
                B=int(n[3], 16)
            # Change Int to float
            R=float(R)/255
            G=float(G)/255
            B=float(B)/255
            # Material flow
            if k==0:
                # have a Material
                mat.R=R
                mat.G=G
                mat.B=B
                me.materials[0]=mat
            else:
                # New a Material
                mat=me.materials[0]
                mat.name=n[0]
                mat.R=R
                mat.G=G
                mat.B=B
            me.update()
            print "        %3d: R=%.3f  G=%.3f  B=%.3f  %s" % (cnt,mat.R,mat.G,mat.B,n[0])
    except:
        ConfirmPopMenu(' error..')
    
    f.close()
    Blender.Redraw()
    
    print '       ',time.time()-start,'\n    end..'

#=========================================================
# DrawEvent
#=========================================================
def DrawGui():
    global InputBtn,ColumnBtn,StepXBtn,StepYBtn,MenuBtn
    # Label
    StrList=['File','>> ','Column: ','Step','X: ','Y: ','Duplicate','Exit']
    StrList_Select	= 0
    StrList_Input	= 1
    StrList_Column	= 2
    StrList_Step	= 3
    StrList_StepX	= 4
    StrList_StepY	= 5
    StrList_Make	= 6
    StrList_Exit	= 7
    #
    SpcWidth=20
    BaseHeight=20
    # --- Init Screen ---
    BGL.glClearColor(0.6, 0.6, 0.6, 0.0)
    BGL.glClear(Blender.BGL.GL_COLOR_BUFFER_BIT)
    BGL.glColor3f(0.1, 0.1, 0.15) # Text Color
    # --- Get Screen Size ---
    size=Blender.BGL.Buffer(Blender.BGL.GL_FLOAT, 4)
    Blender.BGL.glGetFloatv(Blender.BGL.GL_SCISSOR_BOX, size)
    size= size.list
    # FLOAT -> INT
    for s in range(4): size[s]=int(size[s])
    # Base Position
    x=20
    y=size[3]-BaseHeight
    # + Title
    BGL.glRasterPos2f(x, y)
    Draw.Text(Title)
    # + File Select
    y-=BaseHeight*2-10
    w=Draw.GetStringWidth(StrList[StrList_Select])+SpcWidth
    Draw.Button (StrList[StrList_Select],Event_Select,x,y,w,BaseHeight,InputBtn.val)
    InputBtn=Draw.String(StrList[StrList_Input], Event_Input, x+w+4, y, size[2]-w-40-80, BaseHeight, InputBtn.val, 255)
    menuName = "Data Type %t|Decimal %x1|Hex %x2|Html %x3"
    MenuBtn = Draw.Menu(menuName, Event_Menu, (x+w+4)+(size[2]-w-40-80)+6, y, 80, BaseHeight, MenuBtn.val)
    # + Button
    #   - Duplicate
    y-=BaseHeight*2-10
    w=Draw.GetStringWidth(StrList[StrList_Make])+SpcWidth
    Draw.Button (StrList[StrList_Make],Event_Duplicate,x,y,w,BaseHeight,"")
    #   - Column
    i=x+w+SpcWidth
    k=Draw.GetStringWidth(StrList[StrList_Column])+SpcWidth*2+8
    ColumnBtn=Draw.Number(StrList[StrList_Column], Event_Column, i, y, k, BaseHeight, ColumnBtn.val, 1, 99)
    #   - StepX / StepY
    i+=k+SpcWidth
    BGL.glRasterPos2f(i, y+6)
    Draw.Text(StrList[StrList_Step])
    i+=Draw.GetStringWidth(StrList[StrList_Step])+4
    w=Draw.GetStringWidth('100.00')+SpcWidth*2
    StepXBtn=Draw.Number(StrList[StrList_StepX], Event_StepX, i    , y, w, BaseHeight, StepXBtn.val, 1, 100)
    StepYBtn=Draw.Number(StrList[StrList_StepY], Event_StepY, i+w+4, y, w, BaseHeight, StepYBtn.val, 1, 100)
    # + Version
    y-=BaseHeight*2-10
    w=Draw.GetStringWidth(Version+Author)
    BGL.glRasterPos2f(x+size[2]-w-40, y)
    Draw.Text(Version+Author)
    # + Exit Button
    y-=BaseHeight*2-10
    Draw.Button (StrList[StrList_Exit],Event_Exit,x,y,Draw.GetStringWidth(StrList[StrList_Exit])+SpcWidth,BaseHeight,"")

#=========================================================
# KeyEvent
#=========================================================
def KeyEvent(evt, val):
    if (evt == Draw.ESCKEY) or (evt == Draw.QKEY):
        ExitFunction()
        return

#=========================================================
# ButtonEvent
#=========================================================
def ButtonEvent(evt):
    if (evt == Event_Exit)	: ExitFunction()
    elif (evt == Event_Select)	: Window.FileSelector (FileSelectFunction, 'File Select..')
    elif (evt == Event_Duplicate):DuplicateFunction()
#    elif (evt == Event_Menu)	: MenuBtn.val

#=========================================================
# Main
#=========================================================
print '\n',Title,Version,Author
Draw.Register(DrawGui, KeyEvent, ButtonEvent)
